from colorboy import green, cyan, red
from pathlib import Path

package_name = 'vidl' # used for getting config location and package version
package_author = 'Kasper Henningsen' # used for getting config location

def log(*args, error=False, quiet=False, callback=None, quit=True, **named_args):
    vidl_text = cyan('[vidl]')
    if error:
        print(vidl_text, red('Error:'), *args, **named_args)
    elif quiet == False:
        print(vidl_text, *args, **named_args)

script_filename = "vidl"
# used to be sys.argv[0], but Poetry changes sys.argv[0] to the full path
def vidl_help():
    print(          '')
    print(green(    'Download Usage:'))
    print(          '    '+cyan(script_filename)+' [format] [options] <URL>')
    print(          '')
    print(green(    'Download Options:'))
    print(cyan(     '    format             ')+'mp3, mp4, wav or m4a. Default mp3.')
    print(cyan(     '    --no-md            ')+'Don\'t add metadata to downloaded files.')
    print(cyan(     '    --no-smart-md      ')+'Don\'t extract artist and song name from title.')
    print(cyan(     '    --no-dl            ')+'Don\'t download anything. Usually used with -v')
    print(cyan(     '    -v, --verbose      ')+'Display all logs.')
    print(          '')
    print(green(    'Global Options:'))
    print(cyan(     '    --version          ')+'Display vidl version. "vidl -v" also works.')
    print(cyan(     '    -h, --help         ')+'Display this help message.')
    print(cyan(     '    --config-path      ')+'Display the location of vidl\'s configuration file.')
    print(          '')
    quit()

def main():
    import sys
    if len(sys.argv) <= 1 or '--help' in sys.argv or '-h' in sys.argv:
        vidl_help()
    elif '--version' in sys.argv or sys.argv[1:] == ['-v']:
        import vidl.version
        log("Version", vidl.version.get_package_version())
    elif '--config-path' in sys.argv:
        import vidl.config
        log("Config path:", vidl.config.config_path)
    else:
        from vidl import dl
        dl.main()
if __name__ == '__main__':
    main()



import sys, os
import pprint; pprint = pprint.PrettyPrinter(indent=4).pprint
import youtube_dl
from colorboy import cyan, green
from deep_filter import deep_filter

from vidl import app, config
from vidl.app import log

class Dicty(dict):
    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__
def is_int(number):
    try:
        int(number)
        return True
    except ValueError:
        return False

def main():

    options = {
        'url': '',
        'file_format': 'mp3',
        'audio_only': True,
        'no_md': False,
        'no_smart_md': False,
        'no_dl': False,
        'verbose': False,
        'download_folder': config.get_config('download_folder'),
        'output_template': config.get_config('output_template'),
    }
    if options['download_folder'] == None:
        log('download_folder config has not been set. Add a download folder to the vidl config file.', error=True)
        log("Config path:", config.config_path)
        quit()

    video_formats = ['mp4']
    audio_formats = ['mp3', 'wav', 'm4a']
    id3_metadata_formats = ['mp3']
    ytdl_output_template = os.path.join(options['download_folder'], options['output_template'])

    # parse arguments
    for arg in sys.argv[1:]:
        if arg in audio_formats:
            options['audio_only'] = True
            options['file_format'] = arg
        elif arg in video_formats:
            options['audio_only'] = False
            options['file_format'] = arg
        elif arg in ['--no-md']:
            options['no_md'] = True
        elif arg in ['--no-smart-md']:
            options['no_smart_md'] = True
        elif arg in ['--no-dl']:
            options['no_dl'] = True
        elif arg in ['-v', '--verbose']:
            options['verbose'] = True
        elif '.' in arg:
            options['url'] = arg
        else:
            log('Unknown argument:', arg, error=True)
            quit()
    if options['url'] == '':
        log('No URL provided', error=True)
        quit()

    # get info
    log('Fetching URL info')
    ytdl_get_info_options = {
        'outtmpl': ytdl_output_template,
        'quiet': False if options['verbose'] else True,
    }
    with youtube_dl.YoutubeDL(ytdl_get_info_options) as ytdl:
        try:
            info_result = ytdl.extract_info(options['url'], download=False)
        except:
            quit()
        if options['verbose']:
            pprint(info_result)

    # delete None properties/indexes
    def callback(value):
        return value != None
    cleaned_info_result = deep_filter(info_result.copy(), callback)

    # restructure
    url_info = cleaned_info_result.copy()
    if 'entries' in cleaned_info_result:
        videos = cleaned_info_result['entries']
        playlist_info = cleaned_info_result.copy()
        del playlist_info['entries']
    else:
        videos = [cleaned_info_result]
        playlist_info = {}

    # generate ytdl arguments
    ytdl_args = []
    if options['audio_only']:
        ytdl_args += ['-x']
        ytdl_args += ['-f', 'best']
        ytdl_args += ['--audio-format', options['file_format']]
    else:
        ytdl_args += ['-f', 'bestvideo+bestaudio']
        ytdl_args += ['--recode-video', options['file_format']]
    ytdl_args += ['--audio-quality', '0']
    ytdl_args += ['-o', ytdl_output_template]
    if options['file_format'] in ['mp3', 'm4a', 'mp4']:
        ytdl_args += ['--embed-thumbnail']
    if not options['verbose']:
        ytdl_args += ['--quiet']
    # ytdl_args += [options['url']]

    video_index = -1
    for video in videos:
        video_index += 0
        try:
            filename = ytdl.prepare_filename(video)
        except:
            quit()
        filename_split = filename.split('.')
        filename_split[len(filename_split)-1] = options['file_format']
        filename = '.'.join(filename_split)
        if options['verbose']:
            log(green('youtube-dl command:'), 'youtube-dl', ' '.join(ytdl_args+[video['webpage_url']]))
        if options['no_dl']:
            continue
        log('Downloading')
        try:
            youtube_dl.main(ytdl_args+[video['webpage_url']])
        except:
            pass
        log('Saved as', filename)
        if options['file_format'] in id3_metadata_formats and not options['no_md']:
            log('Adding metadata to file')

            # get artist/title from title
            parsed_title = {}
            if not options['no_smart_md']:
                if 'title' in video and video['title'].count(' - ') == 1:
                    split_title = video['title'].split(' - ')
                    parsed_title['artist'] = split_title[0]
                    parsed_title['title'] = split_title[1]

            md = Dicty()
            playlist = True if len(videos) > 1 else False

            # title
            if 'title' in parsed_title:
                smart_title = True
            if 'title' in video:
                md.title = video['title']
            elif 'track' in video:
                md.title = video['track']
            # artist
            if 'artist' in parsed_title:
                smart_artist = True
            if 'uploader' in video:
                md.artist = video['uploader']
            elif 'artist' in video:
                md.artist = video['artist']

            if playlist:
                #album
                if 'title' in playlist_info:
                    md.album = playlist_info['title']
                elif 'playlist_title' in video:
                    md.album = video['playlist_title']
                elif 'playlist' in video and type(video['playlist']) == str:
                    md.album = video['playlist']
                #album_artist
                if 'uploader' in playlist_info:
                    md.album_artist = playlist_info['uploader']
                elif 'playlist_uploader' in video:
                    md.album_artist = video['playlist_uploader']
                # track_number
                if 'playlist_index' in video:
                    md.track_number = video['playlist_index']
                else:
                    md.track_number = video_index+1                   
                # track_count
                if 'n_entries' in video:
                    md.track_count = video['n_entries']
                else:
                    md.track_count = len(videos)
            # year
            if 'release_date' in video and is_int(video['release_date'][:4]):
                md.year = video['release_date'][:4]
            elif 'publish_date' in video and is_int(video['publish_date'][:4]):
                md.year = video['publish_date'][:4]
            elif 'upload_date' in video and is_int(video['upload_date'][:4]):
                md.year = video['upload_date'][:4]
            
            dumb_md = md
            if smart_title: md['title'] = parsed_title['title']
            if smart_artist: md['artist'] = parsed_title['artist']

            md = config.user_md_parser(md, dumb_md, video, url_info)
            
            from vidl import md as md_module
            md_module.add_metadata(filename, md)
    log('Done')



import sys, json, os, appdirs
from colorboy import green
from vidl.app import log, package_name, package_author

user_data_dir = appdirs.user_data_dir(package_name, package_author)
config_path = os.path.join(user_data_dir, 'config.json')
user_md_parser_path = os.path.join(user_data_dir, 'user_md_parser.py')
default_user_md_parser_path = os.path.join(os.path.dirname(__file__), 'default_user_md_parser.py')

def save_file(path, content, json=False):
    try:
        file = open(path, 'w+')
    except FileNotFoundError:
        os.makedirs(user_data_dir)
        file = open(path, 'w+')
    file.write(content)
    file.close()

def get_default_download_folder():
    if sys.platform == 'darwin':
        return '~/Downloads'
    elif sys.platform == 'win32':
        import os
        path = os.path.join(os.getenv('USERPROFILE'), 'Downloads')
        return path
    else:
        return None

default_config = {
    'download_folder': get_default_download_folder(),
    'output_template': '%(uploader)s - %(title)s.%(ext)s',
}

if not os.path.isfile(config_path):
    save_file(config_path, json.dumps(default_config, indent=2))
if not os.path.isfile(user_md_parser_path):
    default_user_md_parser = open(default_user_md_parser_path).read()
    save_file(user_md_parser_path, default_user_md_parser)

configs = json.loads(open(config_path).read())
from importlib.machinery import SourceFileLoader
user_md_parser = SourceFileLoader('user_md_parser', user_md_parser_path).load_module().user_md_parser

def get_config(key):
    if key not in configs:
        log('Config does not exist:', green(key), error=True)
        quit()
    return configs[key]



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['vidl']

package_data = \
{'': ['*']}

install_requires = \
['appdirs>=1.4,<2.0',
 'colorama>=0.3.9,<0.4.0',
 'colorboy>=1.0.1,<2.0.0',
 'deep-filter>=1.0,<2.0',
 'mutagen>=1.41,<2.0',
 'webob>=1.8,<2.0',
 'youtube-dl>=2018.9,<2019.0']

entry_points = \
{'console_scripts': ['vidl = vidl.app:main']}

setup_kwargs = {
    'name': 'vidl',
    'version': '3.4.0',
    'description': 'Python script to download video/audio, built with youtube-dl',
    'long_description': '# vidl\nvidl is a script designed to easily download video/audio from anywere, using youtube-dl. It automatically embeds thumbnails to mp3/mp4/m4a files.\n\nvidl will add metadata to mp3 files if it\'s found. The `--no-md` option turns this off.\n`title`, `artist` and `year` metadata is added, but if the URL is a playlist, it also adds `album`, `album artist`, `track number`, `track count`.\nIf the title contains " - ", vidl often uses what comes before and after it as artist and title respectively. The `--dont-extract-md` option turns off this behaviour.\n\n# Installation\n1. Install Python (3.7 is recommended)\n2. Install [ffmpeg and ffprobe](https://www.ffmpeg.org/)\n3. Run `pip install vidl`\n4. If you\'re not on macOS or Windows, you need to specify where vidl will download files to in your vidl config file. Run `vidl --config-path` to see where the config file is.\nIf you\'re on macOS, I recommend [setting up shortcuts for vidl](#macos-shortcut-setup)\n\n# Uninstall\nRun `pip uninstall vidl`.\n\nIf you want to fully uninstall vidl, go to vidl\'s config file (Run `vidl --config-path` to see where it is) and delete the folder it\'s in.\n\n# Usage\nExamples:\n`vidl https://www.youtube.com/watch?v=ta_ZVS7HkwI`\n- Downloads the video as mp3, and adds metadata it detects.\n\n`vidl mp3 https://www.youtube.com/watch?v=ta_ZVS7HkwI --no-md`\n- Downloads the video as mp3, without adding metadata.\n\n`vidl`\n- Prints vidl\'s help menu, which looks like this:\n    ```\n    Download Usage:\n        vidl [format] [options] <URL>\n\n    Download Options:\n        format             mp3, mp4, wav or m4a. Default mp3.\n        --no-md            Don\'t add metadata to downloaded files.\n        --no-smart-md      Don\'t extract artist and song name from title.\n        --no-dl            Don\'t download anything. Usually used with -v\n        -v, --verbose      Display all logs.\n\n    Global Options:\n        --version          Display vidl version. "vidl -v" also works.\n        -h, --help         Display this help message.\n        --config-path      Display the location of vidl\'s configuration file.\n    ```\n\n# Configuration\nvidl has a configuration file, which you can find the location of by running `vidl --config-path`. In it, you can set the download folder and filename template.\n\nIf you screw something up, you can delete the file, and the default will be recreated the next time you run vidl.\n\n# Custom metadata parsing\nThis is advanced usage. Knowing some programming is useful for doing this.\n\nvidl has a `user_md_parser.py` file. By default, it does nothing, but you can configure it to manipulate metadata of songs you download however you like. In my case, I for use this file to detect if a title contains "[NCS Release]", and if it does I set the "Comment" metadata to "NCS".\n\nDocumentation for this can be found in the file itself. The file is in the same folder as vidl\'s config file, which you can find by by running `vidl --config-path`. If you screw something up, you can delete the file, and the default will be recreated the next time you run vidl.\n\n# <a name="#macos-shortcut-setup"></a>Set up shortcuts for vidl (macOS)\nYou\'ll be able to select any piece of text, press your chosen shortcut and the link(s) in your selected text will be downloaded! A little tedious to set up, but well worth it.\n\nFirst, we need to create a macOS Service:\n1. Open the Automator app.\n2. Choose File > New, and select Service.\n3. (TLDR; Add `Run Shell Script`) In the window that just popped up, there are two columns on the left (if not, click the `Library` button in the status bar). Select `Utilities` in the first column, and in the second column, drag `Run Shell Script` into the main part of the window.\n4. Make your settings match these:\n    \n    ![Service receives selected [URLs] in [any application]. Input is [only URLs]. In your Run Shell Script box; Shell: [/bin/bash]. Pass input: [as arguments]](https://raw.githubusercontent.com/SpectralKH/vidl/master/macos-service-screenshot.png)\n    \n    If you want the shortcut to only work in one app, select that app instead of `any application`.\n5. In the text box in the "Run Shell Script" box, paste in the following script:\n    ```bash\n    for f in "$@"\n    do\n        # AppleScript doesn\'t look for scripts in the same places as the terminal,\n        # so we need to make it look in the proper folders.\n        export PATH=<VIDL_DIR>:$PATH\n        export PATH=<FFMPEG_DIR>:$PATH\n        export PATH=<FFPROBE_DIR>:$PATH\n        vidl --quiet "$f"\n    done\n    ```\n    Replace `<VIDL_DIR>` with the path you get from running `dirname $(which vidl)` in the terminal.\n    Replace `<FFMPEG_DIR>` with the path you get from running `dirname $(which ffmpeg)` in the terminal.\n    Replace `<FFPROBE_DIR>` with the path you get from running `dirname $(which ffprobe)` in the terminal.\n6. Choose File > Save. Type in vidl.\n\nAlmost done, you just need to tie a shortcut to the macOS Service you just created:\n1. Open the System Preferences app.\n2. Go to Keyboard and select the Shortcuts tab.\n3. Select Services from the left column, and locate vidl (should be under Internet). Add your preferred shortcut.\n\n# Dev Instructions\n### Installation\n1. Install Python (3.7 is recommended)\n2. Install [ffmpeg and ffprobe](https://www.ffmpeg.org/)\n3. Install [Poetry](https://poetry.eustace.io)\n4. Run `poetry install` to install Python package dependencies.\n5. If you\'re not on macOS or Windows, you need to specify where vidl will download files to in your vidl config file. Run `vidl --config-path` to see where the config file is.\n\nI recommend running `poetry config settings.virtualenvs.in-project true`. This command makes Poetry create your Python virtual environment inside the project folder, so you\'ll be able to easily delete it. Additionally, it lets VSCode\'s Python extension detect the virtual environment if you set the `python.pythonPath` setting to `${workspaceFolder}/.venv/bin/python` in your workspace (or global) settings.\n\n### Running\nRun `poetry run python vidl`.\n\nTo test the package, you can run `poetry develop`. This adds the `vidl` package itself to the virtual environment, including the CLI executable. This means you can simply type `poetry run vidl`.\n\nAs an alternative to `poetry run <command>`, you can run `poetry shell` to enter the virtual environment\'s bash CLI, and then run your command on it\'s own.\n\n# ToDo\n- For future config possibilities, replace options like `--no-md` with `--md` and `--!md`. Maybe call it `defaults` instead of `config`?\n    - Add all configs as options, for instance add download_folder option.\n    - Add all options as configs, for instance add md option. \n    - Add config/option for individual metadata\n- Allow passing youtube-dl arguments directly\n',
    'author': 'KH',
    'author_email': 'kasperkh.kh@gmail.com',
    'url': 'https://vidl.kasp.io',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.2,<4.0',
}


setup(**setup_kwargs)
